@@ -1,6 +1,8 @@
 extern crate alloc;
 use alloc::borrow::Cow;
 use alloc::collections::BTreeMap;
+use alloc::string::String;
+use alloc::string::ToString;
 
 use ::log::*;
 
@@ -237,7 +239,7 @@ impl<'a> HttpRequest<'a> for EspHttpRequest<'a> {
             }
 
             if self.follow_redirects {
-                let status = unsafe { esp_http_client_get_status_code(self.client.raw) as _ };
+                let status = unsafe { esp_http_client_get_status_code(self.client.raw) as u16 };
 
                 if status::REDIRECT.contains(&status) {
                     info!("Got response {}, about to follow redirect", status);
@@ -293,9 +295,13 @@ impl<'a> Write for EspHttpRequest<'a> {
     type Error = EspError;
 
     fn do_write(&mut self, buf: &[u8]) -> Result<usize, Self::Error> {
-        esp!(unsafe { esp_http_client_write(self.client.raw, buf.as_ptr() as _, buf.len() as _) })?;
+        let result =
+            unsafe { esp_http_client_write(self.client.raw, buf.as_ptr() as _, buf.len() as _) };
+        if result < 0 {
+            esp!(result)?;
+        }
 
-        Ok(buf.len())
+        Ok(result as _)
     }
 }
 
