@@ -24,7 +24,10 @@ pub enum InterfaceStack {
     Sta,
     Ap,
     Eth,
+    #[cfg(esp_idf_ppp_support)]
     Ppp,
+    #[cfg(esp_idf_slip_support)]
+    Slip,
 }
 
 impl InterfaceStack {
@@ -33,7 +36,10 @@ impl InterfaceStack {
             Self::Sta => InterfaceConfiguration::wifi_default_client(),
             Self::Ap => InterfaceConfiguration::wifi_default_router(),
             Self::Eth => InterfaceConfiguration::eth_default_client(),
+            #[cfg(esp_idf_ppp_support)]
             Self::Ppp => InterfaceConfiguration::ppp_default_client(),
+            #[cfg(esp_idf_slip_support)]
+            Self::Slip => InterfaceConfiguration::slip_default_client(),
         }
     }
 }
@@ -102,6 +108,7 @@ impl InterfaceConfiguration {
         }
     }
 
+    #[cfg(esp_idf_ppp_support)]
     pub fn ppp_default_client() -> Self {
         Self {
             key: "PPP_CL_DEF".into(),
@@ -112,6 +119,7 @@ impl InterfaceConfiguration {
         }
     }
 
+    #[cfg(esp_idf_ppp_support)]
     pub fn ppp_default_router() -> Self {
         Self {
             key: "PPP_RT_DEF".into(),
@@ -121,6 +129,28 @@ impl InterfaceConfiguration {
             interface_stack: InterfaceStack::Ppp,
         }
     }
+
+    #[cfg(esp_idf_slip_support)]
+    pub fn slip_default_client() -> Self {
+        Self {
+            key: "SLIP_CL_DEF".into(),
+            description: "slip".into(),
+            route_priority: 35,
+            ip_configuration: InterfaceIpConfiguration::Client(Default::default()),
+            interface_stack: InterfaceStack::Slip,
+        }
+    }
+
+    #[cfg(esp_idf_slip_support)]
+    pub fn slip_default_router() -> Self {
+        Self {
+            key: "SLIP_RT_DEF".into(),
+            description: "sliprt".into(),
+            route_priority: 25,
+            ip_configuration: InterfaceIpConfiguration::Router(Default::default()),
+            interface_stack: InterfaceStack::Slip,
+        }
+    }
 }
 
 static mut TAKEN: EspMutex<(bool, bool)> = EspMutex::new((false, false));
@@ -174,13 +204,13 @@ impl EspNetif {
         let c_if_key = CString::new(conf.key.as_str()).unwrap();
         let c_if_description = CString::new(conf.description.as_str()).unwrap();
 
-        let (mut esp_inherent_config, ip_info, dhcps, dns, secondary_dns) = match conf
+        let (mut esp_inherent_config, ip_info, dhcps, dns, secondary_dns, hostname) = match conf
             .ip_configuration
         {
             InterfaceIpConfiguration::Client(ref ip_conf) => (
                 esp_netif_inherent_config_t {
                     flags: match ip_conf {
-                        ipv4::ClientConfiguration::DHCP => {
+                        ipv4::ClientConfiguration::DHCP(_) => {
                             esp_netif_flags_ESP_NETIF_DHCP_CLIENT
                                 | esp_netif_flags_ESP_NETIF_FLAG_GARP
                                 | esp_netif_flags_ESP_NETIF_FLAG_EVENT_IP_MODIFIED
@@ -192,7 +222,7 @@ impl EspNetif {
                     mac: [0; 6],
                     ip_info: ptr::null(),
                     get_ip_event: match ip_conf {
-                        ipv4::ClientConfiguration::DHCP => {
+                        ipv4::ClientConfiguration::DHCP(_) => {
                             if conf.interface_stack == InterfaceStack::Sta {
                                 ip_event_t_IP_EVENT_STA_GOT_IP
                             } else {
@@ -202,7 +232,7 @@ impl EspNetif {
                         ipv4::ClientConfiguration::Fixed(_) => 0,
                     },
                     lost_ip_event: match ip_conf {
-                        ipv4::ClientConfiguration::DHCP => {
+                        ipv4::ClientConfiguration::DHCP(_) => {
                             if conf.interface_stack == InterfaceStack::Sta {
                                 ip_event_t_IP_EVENT_STA_LOST_IP
                             } else {
@@ -216,7 +246,7 @@ impl EspNetif {
                     route_prio: conf.route_priority as _,
                 },
                 match ip_conf {
-                    ipv4::ClientConfiguration::DHCP => None,
+                    ipv4::ClientConfiguration::DHCP(_) => None,
                     ipv4::ClientConfiguration::Fixed(ref fixed_conf) => Some(esp_netif_ip_info_t {
                         ip: Newtype::<esp_ip4_addr_t>::from(fixed_conf.ip).0,
                         netmask: Newtype::<esp_ip4_addr_t>::from(fixed_conf.subnet.mask).0,
@@ -225,13 +255,17 @@ impl EspNetif {
                 },
                 false,
                 match ip_conf {
-                    ipv4::ClientConfiguration::DHCP => None,
+                    ipv4::ClientConfiguration::DHCP(_) => None,
                     ipv4::ClientConfiguration::Fixed(ref fixed_conf) => fixed_conf.dns,
                 },
                 match ip_conf {
-                    ipv4::ClientConfiguration::DHCP => None,
+                    ipv4::ClientConfiguration::DHCP(_) => None,
                     ipv4::ClientConfiguration::Fixed(ref fixed_conf) => fixed_conf.secondary_dns,
                 },
+                match ip_conf {
+                    ipv4::ClientConfiguration::DHCP(ref dhcp_conf) => dhcp_conf.hostname.as_ref(),
+                    ipv4::ClientConfiguration::Fixed(_) => None,
+                },
             ),
             InterfaceIpConfiguration::Router(ref ip_conf) => (
                 esp_netif_inherent_config_t {
@@ -256,6 +290,7 @@ impl EspNetif {
                 ip_conf.dhcp_enabled,
                 ip_conf.dns,
                 None, /* For APs, ESP-IDF supports setting a primary DNS only ip_conf.secondary_dns */
+                None,
             ),
         };
 
@@ -271,10 +306,10 @@ impl EspNetif {
                     InterfaceStack::Sta => _g_esp_netif_netstack_default_wifi_sta,
                     InterfaceStack::Ap => _g_esp_netif_netstack_default_wifi_ap,
                     InterfaceStack::Eth => _g_esp_netif_netstack_default_eth,
-                    #[cfg(any(esp_idf_version = "4.4", esp_idf_version_major = "5"))]
-                    InterfaceStack::Ppp => panic!("Not yet supported on ESP-IDF 4.4 & ESP-IDF 5.X"),
-                    #[cfg(not(any(esp_idf_version = "4.4", esp_idf_version_major = "5")))]
+                    #[cfg(esp_idf_ppp_support)]
                     InterfaceStack::Ppp => _g_esp_netif_netstack_default_ppp,
+                    #[cfg(esp_idf_slip_support)]
+                    InterfaceStack::Slip => _g_esp_netif_netstack_default_slip,
                 }
             },
         };
@@ -303,6 +338,10 @@ impl EspNetif {
             netif.set_secondary_dns(secondary_dns);
         }
 
+        if let Some(hostname) = hostname {
+            netif.set_hostname(hostname);
+        }
+
         Ok(netif)
     }
 
@@ -392,6 +431,20 @@ impl EspNetif {
             )
         };
     }
+
+    pub fn get_hostname(&self) -> Result<Cow<'_, str>, EspError> {
+        let mut ptr: *const c_types::c_char = std::ptr::null();
+        esp!(unsafe { esp_netif_get_hostname(self.1, &mut ptr) })?;
+        Ok(from_cstr_ptr(ptr))
+    }
+
+    pub fn set_hostname(&self, hostname: &str) -> Result<(), EspError> {
+        if let Ok(hostname) = CString::new(hostname) {
+            esp!(unsafe { esp_netif_set_hostname(self.1, hostname.as_ptr() as *const _) })
+        } else {
+            esp!(ESP_ERR_INVALID_ARG)
+        }
+    }
 }
 
 impl Drop for EspNetif {
